@@ -67,7 +67,6 @@ Get the member args for an operations::
 
 """
 import os
-import sys
 import glob
 import logging
 
@@ -129,7 +128,7 @@ def _load_nested_data(session, data_path):
     mod_names = data_path.split('/')
     for i, md in enumerate(mod_names):
         mod_name = '/'.join(mod_names[0:i + 1])
-        d = _load_data(session, mod_name)
+        _load_data(session, mod_name)
     data = None
     prefixes = [dp for dp in _data_cache.keys() if data_path.startswith(dp)]
     if prefixes:
@@ -170,7 +169,7 @@ def get_search_path(session):
             os.path.dirname(
                 os.path.abspath(__file__))), 'botocore', 'data')
     paths = [builtin_path]
-    search_path = session.get_variable('data_path')
+    search_path = session.get_config_variable('data_path')
     if search_path is not None:
         extra_paths = search_path.split(os.pathsep)
         for path in extra_paths:
