@@ -1,5 +1,5 @@
-# Copyright (c) 2012 Mitch Garnaat http://garnaat.org/
-# Copyright 2012 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright (c) 2012-2013 Mitch Garnaat http://garnaat.org/
+# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -28,7 +28,7 @@ import botocore.exceptions
 
 def get_config(session):
     """
-    If the 'AWS_CONFIG_FILE' environment variable exists, parse that
+    If the ``config_file`` session variable exists, parse that
     file and return all of the data found within the file as a
     dictionary of dictionaries, one for each profile section found
     in the configuration file.
@@ -41,9 +41,8 @@ def get_config(session):
     """
     config = {}
     path = None
-    env_var = session.env_vars['config_file']
-    if env_var in os.environ:
-        path = os.getenv(env_var)
+    path = session.get_config_variable('config_file')
+    if path is not None:
         path = os.path.expandvars(path)
         path = os.path.expanduser(path)
         if not os.path.isfile(path):
