@@ -24,6 +24,7 @@ import logging
 
 from .endpoint import get_endpoint
 from .operation import Operation
+from .waiter import Waiter
 from .exceptions import ServiceNotInRegionError, NoRegionError
 
 
@@ -42,6 +43,7 @@ class Service(object):
         optional value is an endpoint for that region.
     :ivar protocols: A list of protocols supported by the service.
     """
+    WAITER_CLASS = Waiter
 
     def __init__(self, session, provider, service_name,
                  path='/', port=None):
@@ -97,7 +99,7 @@ class Service(object):
         return endpoint_url
 
     def get_endpoint(self, region_name=None, is_secure=True,
-                     endpoint_url=None):
+                     endpoint_url=None, verify=None):
         """
         Return the Endpoint object for this service in a particular
         region.
@@ -120,16 +122,16 @@ class Service(object):
 
         """
         if region_name is None:
-            region_name = self.session.get_variable('region')
+            region_name = self.session.get_config_variable('region')
         if endpoint_url is not None:
             # Before getting into any of the region/endpoint
             # logic, if an endpoint_url is explicitly
             # provided, just use what's been explicitly passed in.
-            return get_endpoint(self, region_name, endpoint_url)
+            return self._get_endpoint(region_name, endpoint_url, verify)
         if region_name is None and not self.global_endpoint:
             # The only time it's ok to *not* provide a region is
             # if the service is a global_endpoint (e.g. IAM).
-            envvar_name = self.session.env_vars['region'][1]
+            envvar_name = self.session.session_var_map['region'][1]
             raise NoRegionError(env_var=envvar_name)
         if region_name not in self.region_names:
             if self.global_endpoint:
@@ -161,7 +163,14 @@ class Service(object):
             # endpoint_prefix.region.amazonaws.com.
             host = '%s.%s.amazonaws.com' % (self.endpoint_prefix, region_name)
             endpoint_url = self._build_endpoint_url(host, is_secure)
-        return get_endpoint(self, region_name, endpoint_url)
+        return self._get_endpoint(region_name, endpoint_url, verify)
+
+    def _get_endpoint(self, region_name, endpoint_url, verify):
+        event = self.session.create_event('creating-endpoint',
+                                          self.endpoint_prefix)
+        self.session.emit(event, service=self, region_name=region_name,
+                          endpoint_url=endpoint_url)
+        return get_endpoint(self, region_name, endpoint_url, verify)
 
     def get_operation(self, operation_name):
         """
@@ -178,6 +187,13 @@ class Service(object):
                 return operation
         return None
 
+    def get_waiter(self, waiter_name):
+        if waiter_name not in self.waiters:
+            raise ValueError("Waiter does not exist: %s" % waiter_name)
+        config = self.waiters[waiter_name]
+        operation = self.get_operation(config['operation'])
+        return self.WAITER_CLASS(waiter_name, operation, config)
+
 
 def get_service(session, service_name, provider):
     """
